@@ -7,7 +7,7 @@ import pkgutil
 
 
 __package__ = "mayatk"
-__version__ = "0.9.24"
+__version__ = "0.9.35"
 
 """Dynamic Attribute Resolver for Module-based Packages
 
@@ -44,26 +44,45 @@ IMPORTED_MODULES = {}
 MODULE_TO_PARENT = {}
 
 
-def build_dictionaries(included_modules=None):
+# Optimization 2: More efficient method detection
+def _add_class_methods(class_obj, module_name, class_name):
+    """Helper function to add class methods to the method dictionaries."""
+    # Use __dict__ for faster iteration on user-defined methods
+    for method_name in class_obj.__dict__:
+        method = getattr(class_obj, method_name)
+        if callable(method) and not method_name.startswith("_"):
+            METHOD_TO_MODULE[method_name] = (module_name, class_name)
+            CLASS_METHOD_TO_MODULE[method_name] = (module_name, class_name)
+
+
+def build_dictionaries(include=None):
     base_path = os.path.dirname(__file__)
     base_package = __name__
 
     if base_package == "__main__":
         raise EnvironmentError("build_dictionaries cannot be run as a script.")
 
+    include = include or {}
+
+    # Pre-compute nested module paths for efficiency
+    nested_paths = {
+        f"{base_package}.{key}": (key, classes)
+        for key, classes in include.items()
+        if "." in key
+    }
+
     for importer, modname, ispkg in pkgutil.walk_packages(
         path=[base_path], prefix=base_package + "."
     ):
         module_name_component = modname.split(".")[-1]
 
-        # If included_modules is defined, skip modules not in the list
-        if (
-            included_modules is not None
-            and module_name_component not in included_modules
-        ):
-            parent_package = modname.rsplit(".", 1)[0] if "." in modname else None
-            if parent_package:
-                MODULE_TO_PARENT[module_name_component] = parent_package
+        # Check direct match first (most common case)
+        if module_name_component in include:
+            classes_to_include = include[module_name_component]
+        # Check pre-computed nested paths
+        elif modname in nested_paths:
+            _, classes_to_include = nested_paths[modname]
+        else:
             continue
 
         try:
@@ -72,18 +91,19 @@ def build_dictionaries(included_modules=None):
             print(f"Failed to import module {modname}: {e}")
             continue
 
-        for name, obj in inspect.getmembers(module, inspect.isclass):
-            if obj.__module__ == modname:
-                CLASS_TO_MODULE[name] = modname
-                method_members = inspect.getmembers(
-                    obj,
-                    lambda member: inspect.isfunction(member)
-                    or inspect.ismethod(member),
-                )
-                for method_name, _ in method_members:
-                    # Method detection refinement needed here based on actual method classification
-                    METHOD_TO_MODULE[method_name] = (modname, name)
-                    CLASS_METHOD_TO_MODULE[method_name] = (modname, name)
+        # Handle wildcard - expose all classes
+        if "*" in classes_to_include:
+            for name, obj in inspect.getmembers(module, inspect.isclass):
+                if obj.__module__ == modname:
+                    CLASS_TO_MODULE[name] = modname
+                    _add_class_methods(obj, modname, name)
+        else:
+            # Handle specific class names
+            for class_name in classes_to_include:
+                obj = getattr(module, class_name, None)
+                if obj and inspect.isclass(obj) and obj.__module__ == modname:
+                    CLASS_TO_MODULE[class_name] = modname
+                    _add_class_methods(obj, modname, class_name)
 
 
 def import_module(module_name):
@@ -119,22 +139,38 @@ def __getattr__(name):
 
 
 # --------------------------------------------------------------------------------------------
-# Classes and methods from these modules will be exposed at package level.
-included_modules = [
-    "_core_utils",
-    "components",
-    "_env_utils",
-    "_display_utils",
-    "_edit_utils",
-    "_mat_utils",
-    "_anim_utils",
-    "_cam_utils",
-    "_node_utils",
-    "_rig_utils",
-    "_xform_utils",
-    "_ui_utils",
-]
-build_dictionaries(included_modules=included_modules)
+# Unified include dictionary supporting both simple modules and nested module paths
+include = {
+    # Legacy modules - expose all classes using wildcard
+    "_anim_utils": ["*"],
+    "_cam_utils": ["*"],
+    "_core_utils": ["*"],
+    "_display_utils": ["*"],
+    "_edit_utils": ["*"],
+    "_env_utils": ["*"],
+    "_mat_utils": ["*"],
+    "_node_utils": ["*"],
+    "_rig_utils": ["*"],
+    "_ui_utils": ["*"],
+    "_uv_utils": ["*"],
+    "_xform_utils": ["*"],
+    # Specific classes from modules
+    "components": ["Components"],
+    "macros": ["Macros"],
+    "maya_menu_handler": ["MayaMenuHandler"],
+    "naming": ["Naming"],
+    "ui_manager": ["UiManager"],
+    # Selection utilities
+    "edit_utils.selection": ["Selection"],
+    # Add hierarchy manager support (these will now work!):
+    "env_utils.hierarchy_manager.manager": ["HierarchyManager"],
+    "env_utils.hierarchy_manager.core": ["DiffResult", "RepairAction", "FileFormat"],
+    "env_utils.hierarchy_manager.swapper": ["ObjectSwapper"],
+    # Examples of wildcard usage:
+    # "some_module": ["*"],  # Expose all classes from some_module
+}
+
+build_dictionaries(include=include)
 
 # --------------------------------------------------------------------------------------------
 # Notes
