@@ -846,7 +846,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
         return vertPairs
 
     @classmethod
-    @core_utils.CoreUtils.undo
+    @core_utils.CoreUtils.undoable
     def get_closest_vertex(
         cls, vertices, obj, tolerance=0.0, freeze_transforms=False, returned_type="str"
     ):
@@ -997,7 +997,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
         return dist
 
     @staticmethod
-    @core_utils.CoreUtils.undo
+    @core_utils.CoreUtils.undoable
     def bridge_connected_edges(edges: Union[str, object, list]) -> None:
         """Bridges two connected edges by extruding one edge, then moving and merging
         the new vertices with the corresponding vertices of the second edge.
@@ -1352,7 +1352,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
         return filtered_edges
 
     @classmethod
-    @core_utils.CoreUtils.undo
+    @core_utils.CoreUtils.undoable
     def set_edge_hardness(
         cls,
         objects: Union[str, object, List],
@@ -1461,7 +1461,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
         return similar_faces
 
     @classmethod
-    @core_utils.CoreUtils.undo
+    @core_utils.CoreUtils.undoable
     def average_normals(cls, objects, by_uv_shell=False):
         """Average the normals of the given objects.
 
@@ -1485,7 +1485,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
                     pm.polySoftEdge(obj, a=180)
 
     @staticmethod
-    @core_utils.CoreUtils.undo
+    @core_utils.CoreUtils.undoable
     def transfer_normals(objects: list[str], space: str = "world"):
         """Transfer vertex normals from source mesh to target meshes.
 
@@ -1493,29 +1493,23 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
             objects (list): List of mesh names, with the first being the source and the rest being the targets.
             space (str): The space in which to transfer the normals ('world' or 'local').
         """
-        if len(objects) < 2:
-            raise ValueError(
-                "At least one source and one target mesh must be provided."
-            )
-
         space_map = {"world": 0, "local": 1, "component": 4, "topology": 5}
         if space not in space_map:
             valid_spaces = ", ".join(space_map.keys())
             raise ValueError(f"space parameter must be one of: {valid_spaces}")
 
+        # Filter objects to ensure only polygonal meshes are included
+        objs = pm.ls(objects, type="mesh")
+        if len(objs) < 2:
+            raise ValueError(
+                "At least one source and one target mesh must be polygonal meshes."
+            )
+
+        source_mesh, *target_meshes = objs
         sample_space_value = space_map[space]
 
-        source_mesh, *target_meshes = pm.ls(objects, flatten=True)
-
-        # Ensure we are working with shape nodes
-        if isinstance(source_mesh, pm.nt.Transform):
-            source_mesh = source_mesh.getShape()
-
+        source_vertices = source_mesh.numVertices()
         for target_mesh in target_meshes:
-            if isinstance(target_mesh, pm.nt.Transform):
-                target_mesh = target_mesh.getShape()
-
-            source_vertices = source_mesh.numVertices()
             target_vertices = target_mesh.numVertices()
 
             if source_vertices != target_vertices:
@@ -1529,7 +1523,7 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
                 target_mesh,
                 transferNormals=1,
                 sampleSpace=sample_space_value,
-                searchMethod=3,  # closest to point
+                searchMethod=3,  # Closest to point
                 colorBorders=1,
             )
 
@@ -1634,246 +1628,6 @@ class Components(GetComponentsMixin, ptk.HelpMixin):
 
         return normal_vector
 
-    @staticmethod
-    def orient_shells(objects):
-        """Rotate UV shells to run parallel with the most adjacent U or V axis of their bounding box.
-
-        Parameters:
-            objects (str/obj/list): Polygon mesh objects and/or components.
-        """
-        for obj in pm.ls(objects, objectsOnly=True):
-            # filter components for only this object.
-            obj_compts = [i for i in objects if obj in pm.ls(i, objectsOnly=1)]
-            pm.polyLayoutUV(
-                obj_compts,
-                flipReversed=0,
-                layout=0,
-                layoutMethod=1,
-                percentageSpace=0.2,
-                rotateForBestFit=3,
-                scale=0,
-                separate=0,
-            )
-
-    @classmethod
-    def move_to_uv_space(cls, objects, u, v, relative=True):
-        """Move objects to the given u and v coordinates.
-
-        Parameters:
-            objects (str/obj/list): The object(s) to move.
-            u (int): u coordinate.
-            v (int): v coordinate.
-            relative (bool): Move relative or absolute.
-        """
-        # Convert the objects to UVs
-        uvs = pm.polyListComponentConversion(objects, fromFace=True, toUV=True)
-        uvs = pm.ls(uvs, flatten=True)
-
-        # Move the UVs to the given u and v coordinates
-        pm.polyEditUV(uvs, u=u, v=v, relative=relative)
-
-    @classmethod
-    def get_uv_shell_sets(cls, objects=None, returned_type="shell"):
-        """Get UV shells and their corresponding sets of faces.
-
-        Parameters:
-            objects (obj/list): Polygon object(s) or Polygon face(s).
-            returned_type (str): The desired returned type. valid values are: 'shell', 'id'. If None is given, the full dict will be returned.
-
-        Returns:
-            (list)(dict) dependent on the given returned_type arg. ex. {0L:[[MeshFace(u'pShape.f[0]'), MeshFace(u'pShape.f[1]')], 1L:[[MeshFace(u'pShape.f[2]'), MeshFace(u'pShape.f[3]')]}
-        """
-        faces = cls.get_components(objects, "faces", flatten=True)
-
-        shells = {}
-        for face in faces:
-            shell_Id = pm.polyEvaluate(face, uvShellIds=True)
-
-            try:
-                shells[shell_Id[0]].append(face)
-            except KeyError:
-                try:
-                    shells[shell_Id[0]] = [face]
-                except IndexError:
-                    pm.warning(f"Unable to get UV shell ID for face: {face}")
-
-        if returned_type == "shell":
-            return list(shells.values())
-        elif returned_type == "id":
-            return list(shells.keys())
-        else:
-            raise ValueError(
-                f"Invalid returned_type: {returned_type}. Valid values are: 'shell', 'id'."
-            )
-
-    @staticmethod
-    def get_uv_shell_border_edges(objects):
-        """Get the edges that make up any UV islands of the given objects.
-
-        Parameters:
-            objects (str/obj/list): Polygon objects, mesh UVs, or Edges.
-
-        Returns:
-            (list): UV border edges.
-        """
-        uv_border_edges = []
-        for obj in pm.ls(objects):
-            # If the obj is a mesh object, get its shape
-            if isinstance(obj, pm.nt.Transform):
-                obj = obj.getShape()
-
-            # If the obj is a mesh shape, get its UV borders
-            if isinstance(obj, pm.nt.Mesh):
-                # Get the connected edges to the selected UVs
-                connected_edges = pm.polyListComponentConversion(
-                    obj, fromUV=True, toEdge=True
-                )
-                connected_edges = pm.ls(connected_edges, flatten=True)
-            elif isinstance(obj, pm.general.MeshEdge):
-                # If the object is already an edge, no conversion is necessary
-                connected_edges = pm.ls(obj, flatten=True)
-            elif isinstance(obj, pm.general.MeshUV):
-                # If the object is a UV, convert it to its connected edges
-                connected_edges = pm.polyListComponentConversion(
-                    obj, fromUV=True, toEdge=True
-                )
-                connected_edges = pm.ls(connected_edges, flatten=True)
-            else:
-                raise ValueError(f"Unsupported object type: {type(obj)}")
-
-            for edge in connected_edges:
-                edge_uvs = pm.ls(
-                    pm.polyListComponentConversion(edge, tuv=True), fl=True
-                )
-                edge_faces = pm.ls(
-                    pm.polyListComponentConversion(edge, tf=True), fl=True
-                )
-                if (
-                    len(edge_uvs) > 2 or len(edge_faces) < 2
-                ):  # If an edge has more than two uvs or less than 2 faces, it's a uv border edge.
-                    uv_border_edges.append(edge)
-
-        return uv_border_edges
-
-    @staticmethod
-    def get_texel_density(objects, map_size):
-        """Calculate the texel density for the given objects' faces.
-
-        Parameters:
-            objects (str, obj, list): List of mesh objects or a single mesh object to calculate texel density for.
-            map_size (int): Size of the map to calculate the texel density against.
-
-        Returns (float):
-            The texel density.
-        """
-        from math import sqrt
-
-        area_3d_sum = 0.0
-        area_uv_sum = 0.0
-
-        # Convert objects to faces if they are not already
-        if not isinstance(objects, list):
-            objects = [objects]
-        faces = pm.polyListComponentConversion(objects, toFace=True)
-        faces = pm.filterExpand(
-            faces, ex=True, sm=34
-        )  # Now this will work, as faces are passed
-
-        if not faces:
-            pm.warning("No faces found in the input objects.")
-            return 0
-
-        # Calculate 3D and UV areas
-        for f in faces:
-            world_face_area = pm.polyEvaluate(f, worldFaceArea=True)
-            uv_face_area = pm.polyEvaluate(f, uvFaceArea=True)
-            if (
-                world_face_area and uv_face_area
-            ):  # Check if the area lists are not empty
-                area_3d_sum += world_face_area[0]
-                area_uv_sum += uv_face_area[0]
-
-        # Avoid division by zero
-        if area_3d_sum == 0 or area_uv_sum == 0:
-            pm.warning("Cannot calculate texel density with zero area.")
-            return 0
-
-        # Calculate texel density
-        texel_density = (sqrt(area_uv_sum) / sqrt(area_3d_sum)) * map_size
-        return texel_density
-
-    @classmethod
-    def set_texel_density(cls, objects, density, map_size):
-        """Set the texel density for the given objects.
-
-        Parameters:
-            objects (str, obj, list): List of objects or a single object to set texel density for.
-            density (float): The desired texel density.
-            map_size (int): Size of the map to calculate the texel density against.
-        """
-        # Get UV shell sets
-        shells = cls.get_uv_shell_sets(objects, returned_type="shell")
-
-        for shell_faces in shells:
-            # Convert face list to UVs
-            shell_uvs = pm.polyListComponentConversion(shell_faces, toUV=True)
-            shell_uvs = pm.ls(shell_uvs, flatten=True)  # Flatten the list of UVs
-
-            # Calculate current density and scaling factor
-            current_density = cls.get_texel_density(shell_faces, map_size)
-            if current_density == 0:
-                pm.warning(
-                    f"Cannot set texel density for UV shell with zero area: {shell_faces}"
-                )
-                continue  # Skip this shell and continue with the next one
-
-            scale = density / current_density
-
-            # Calculate bounding box center for UVs
-            bc = pm.polyEvaluate(shell_uvs, bc2=True)
-            pU = (bc[0][0] + bc[1][0]) / 2
-            pV = (bc[0][1] + bc[1][1]) / 2
-
-            # Scale UVs
-            pm.polyEditUV(shell_uvs, pu=pU, pv=pV, su=scale, sv=scale)
-
-    @staticmethod
-    def transfer_uvs(
-        source: Union[str, object, List[Union[str, object]]],
-        target: Union[str, object, List[Union[str, object]]],
-        tolerance: float = 0.1,
-    ) -> None:
-        """Transfers UVs from source meshes to target meshes based on geometric similarity. This method is
-        topology-agnostic and can work with different mesh structures.
-
-        Parameters:
-            source (Union[str, pm.nt.Transform, List[Union[str, pm.nt.Transform]]]): The source mesh(es) from
-                which to transfer UVs. Can be a string name, a PyNode object, or a list of these.
-            target (Union[str, pm.nt.Transform, List[Union[str, pm.nt.Transform]]]): The target mesh(es) to
-                which UVs will be transferred. Can be a string name, a PyNode object, or a list of these.
-            tolerance (float): The geometric similarity tolerance within which UV transfer should occur.
-                Defaults to 0.1.
-        """
-        mapping = core_utils.CoreUtils.build_mesh_similarity_mapping(
-            source, target, tolerance
-        )
-        for source_name, target_name in mapping.items():
-            pm.transferAttributes(
-                source_name,
-                target_name,
-                transferPositions=False,
-                transferNormals=False,
-                transferUVs=2,
-                transferColors=0,
-                sampleSpace=4,
-                sourceUvSpace="map1",
-                targetUvSpace="map1",
-                searchMethod=3,
-                flipUVs=False,
-                colorBorders=True,
-            )
-            pm.delete(target_name, ch=True)  # Clean up history on target
-
     @staticmethod
     def crease_edges(edges=None, amount=None, angle=None):
         """Adjust properties of the given edges with optional crease and edge softening/hardening.
