@@ -2,6 +2,8 @@
 # coding=utf-8
 
 from mayatk.edit_utils._edit_utils import *
+from mayatk.edit_utils.selection import *
+from mayatk.edit_utils.naming import *
 
 # --------------------------------------------------------------------------------------------
 
