@@ -12,6 +12,7 @@ import pythontk as ptk
 from mayatk.core_utils import CoreUtils
 from mayatk.node_utils import NodeUtils
 from mayatk.edit_utils import EditUtils
+from mayatk.mat_utils import MatUtils
 from mayatk.display_utils import DisplayUtils
 from mayatk.ui_utils import UiUtils
 
@@ -680,12 +681,17 @@ class EditMacros:
     """ """
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     @CoreUtils.selected
     @CoreUtils.reparent
     @DisplayUtils.add_to_isolation
-    def m_combine(objects):
-        """Combine multiple meshes"""
+    def m_combine(objects, allow_multiple_mats: bool = True):
+        """Combine multiple meshes.
+
+        Parameters:
+            objects (list): List of mesh objects to combine.
+            allow_multiple_materials (bool): If False, abort if selected objects use different materials.
+        """
         if not objects or len(objects) < 2:
             pm.inViewMessage(
                 statusMessage="<hl>Insufficient selection.</hl> Operation requires at least two objects",
@@ -694,13 +700,21 @@ class EditMacros:
             )
             return None
 
+        if not allow_multiple_mats:
+            all_mats = MatUtils.get_mats(objects)
+            if len(set(all_mats)) > 1:
+                pm.warning(
+                    "Cannot combine: selected objects do not share the same material."
+                )
+                return None
+
         combined_mesh = pm.polyUnite(objects, centerPivot=True, ch=False)[0]
         combined_mesh = pm.rename(combined_mesh, objects[0].name())
 
         return combined_mesh
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     @CoreUtils.selected
     @CoreUtils.reparent
     @DisplayUtils.add_to_isolation
@@ -891,10 +905,11 @@ class EditMacros:
     @CoreUtils.selected
     def m_group(objects) -> None:
         """Group selected object(s)."""
+        objects = pm.ls(objects, objectsOnly=True)
         if objects:
             grp = pm.group(objects)
             pm.xform(grp, centerPivots=True)
-            pm.rename(grp, objects[0])
+            pm.rename(grp, objects[0].name())
         else:  # If nothing selected, create empty group.
             pm.group(empty=True, name="null")
 
@@ -928,6 +943,22 @@ class SelectionMacros:
         pm.selectMode(component=True)
         pm.selectType(facet=True)
 
+    @staticmethod
+    def m_invert_selection() -> None:
+        """Invert the current selection of geometry or components."""
+        objects = pm.ls(selection=True, flatten=True)
+
+        if not objects:
+            pm.warning("No valid objects selected to invert.")
+            return
+
+        first = objects[0]
+
+        if isinstance(first, (pm.MeshVertex, pm.MeshEdge, pm.MeshFace)):
+            EditUtils.invert_components(select=True)
+        else:
+            EditUtils.invert_geometry(select=True)
+
     @staticmethod
     @CoreUtils.selected
     def m_toggle_selectability(objects):
@@ -941,29 +972,62 @@ class SelectionMacros:
             return
 
         for obj in pm.ls(objects, flatten=True):
-            # Check the current selectability state using PyNode attributes
-            override_enabled = obj.overrideEnabled.get()
-            current_state = obj.overrideDisplayType.get()
+            try:
+                # Ensure attributes exist and are not locked or connected before modifying
+                if not obj.hasAttr("overrideEnabled") or obj.overrideEnabled.isLocked():
+                    pm.warning(
+                        f"Cannot modify overrideEnabled for {obj}: Attribute is locked."
+                    )
+                    continue
+                if (
+                    not obj.hasAttr("overrideDisplayType")
+                    or obj.overrideDisplayType.isLocked()
+                ):
+                    pm.warning(
+                        f"Cannot modify overrideDisplayType for {obj}: Attribute is locked."
+                    )
+                    continue
+                if (
+                    not obj.hasAttr("useOutlinerColor")
+                    or obj.useOutlinerColor.isLocked()
+                ):
+                    pm.warning(
+                        f"Cannot modify useOutlinerColor for {obj}: Attribute is locked."
+                    )
+                    continue
+                if not obj.hasAttr("outlinerColor") or obj.outlinerColor.isLocked():
+                    pm.warning(
+                        f"Cannot modify outlinerColor for {obj}: Attribute is locked."
+                    )
+                    continue
 
-            if override_enabled and current_state == 2:
-                # Object is currently non-selectable, make it selectable
-                obj.overrideDisplayType.set(0)  # Normal mode
-                obj.useOutlinerColor.set(0)  # Disable custom outliner color
-                pm.inViewMessage(
-                    statusMessage=f"{obj} <hl>Selectable</hl>.",
-                    pos="topCenter",
-                    fade=True,
-                )
-            else:  # Object is currently selectable, make it non-selectable
-                obj.overrideEnabled.set(1)
-                obj.overrideDisplayType.set(2)  # Reference mode
-                obj.useOutlinerColor.set(1)  # Enable custom outliner color
-                obj.outlinerColor.set(0.3, 0.6, 0.6)  # Set color to desaturated teal
-                pm.inViewMessage(
-                    statusMessage=f"{obj} <hl>Non-selectable</hl>.",
-                    pos="topCenter",
-                    fade=True,
-                )
+                override_enabled = obj.overrideEnabled.get()
+                current_state = obj.overrideDisplayType.get()
+
+                if override_enabled and current_state == 2:
+                    # Object is currently non-selectable, make it selectable
+                    obj.overrideDisplayType.set(0)  # Normal mode
+                    obj.useOutlinerColor.set(0)  # Disable custom outliner color
+                    pm.inViewMessage(
+                        statusMessage=f"{obj} <hl>Selectable</hl>.",
+                        pos="topCenter",
+                        fade=True,
+                    )
+                else:  # Object is currently selectable, make it non-selectable
+                    obj.overrideEnabled.set(1)
+                    obj.overrideDisplayType.set(2)  # Reference mode
+                    obj.useOutlinerColor.set(1)  # Enable custom outliner color
+                    obj.outlinerColor.set(
+                        0.3, 0.6, 0.6
+                    )  # Set color to desaturated teal
+                    pm.inViewMessage(
+                        statusMessage=f"{obj} <hl>Non-selectable</hl>.",
+                        pos="topCenter",
+                        fade=True,
+                    )
+
+            except RuntimeError as e:
+                pm.warning(f"Failed to modify selectability for {obj}: {e}")
 
     @staticmethod
     def m_toggle_UV_select_type() -> None:
@@ -1026,9 +1090,9 @@ class UiMacros:
     @staticmethod
     def m_tentacle_show() -> None:
         """Display the tentacle marking menu."""
-        from tentacle import tcl_maya
+        from tentacle.tcl_maya import TclMaya
 
-        tcl_maya.show(key_show="Key_F12")
+        TclMaya.instance().show()
 
     @staticmethod
     def m_toggle_panels(toggle_menu: bool = True, toggle_panels: bool = True) -> None:
