@@ -2,6 +2,12 @@
 # coding=utf-8
 
 from mayatk.env_utils._env_utils import *
+from mayatk.env_utils.workspace_manager import WorkspaceManager
+from mayatk.env_utils.workspace_map import (
+    WorkspaceMap,
+    WorkspaceMapController,
+    WorkspaceMapSlots,
+)
 
 # --------------------------------------------------------------------------------------------
 
