@@ -1,7 +1,8 @@
 # !/usr/bin/python
 # coding=utf-8
 import os
-from typing import Dict, ClassVar, Optional, Any
+import sys
+from typing import Dict, ClassVar, Optional, Union, Any
 
 try:
     import pymel.core as pm
@@ -9,9 +10,6 @@ except ImportError as error:
     print(__file__, error)
 import pythontk as ptk
 
-# From this package:
-from mayatk.core_utils import CoreUtils
-
 
 class EnvUtils(ptk.HelpMixin):
     """ """
@@ -28,7 +26,7 @@ class EnvUtils(ptk.HelpMixin):
     }
 
     @staticmethod
-    def get_maya_info(key):
+    def get_env_info(key):
         """Fetch specific information about the current Maya environment based on the provided key.
 
         Parameters:
@@ -54,6 +52,7 @@ class EnvUtils(ptk.HelpMixin):
             "scene": lambda: pm.sceneName(),
             "scene_name": lambda: ptk.format_path(pm.sceneName(), "name"),
             "scene_path": lambda: ptk.format_path(pm.sceneName(), "path"),
+            "scene_modified": lambda: bool(pm.mel.eval("file -q -modified")),
             "user_name": lambda: pm.optionVar(q="PTglobalUserName"),
             "ui_language": lambda: pm.about(uiLanguage=True),
             "os_type": lambda: pm.about(os=True),
@@ -62,6 +61,12 @@ class EnvUtils(ptk.HelpMixin):
             "loaded_plugins": lambda: pm.pluginInfo(q=True, listPlugins=True),
             "api_version": lambda: pm.about(api=True),
             "host_name": lambda: pm.about(hostName=True),
+            "batch_mode": lambda: pm.about(batch=True),
+            "build_dir": lambda: pm.about(buildDirectory=True),
+            "build_version": lambda: pm.about(buildVersion=True),
+            "build_varient": lambda: pm.about(buildVariant=True),
+            "api_version": lambda: pm.about(apiVersion=True),
+            "application": lambda: pm.about(application=True),
             "current_frame": lambda: pm.currentTime(q=True),
             "frame_range": lambda: (
                 pm.playbackOptions(q=True, min=True),
@@ -174,7 +179,7 @@ class EnvUtils(ptk.HelpMixin):
             plugin_name (str): The name of the plugin to load.
 
         Examples:
-            >>> load_plugin('nearestPointOnMesh')
+            load_plugin('nearestPointOnMesh')
 
         Raises:
             ValueError: If the plugin is not found or fails to load.
@@ -185,6 +190,35 @@ class EnvUtils(ptk.HelpMixin):
             except RuntimeError as e:
                 raise ValueError(f"Failed to load plugin {plugin_name}: {e}")
 
+    @staticmethod
+    def vray_plugin(load=False, unload=False, query=False):
+        """Load/Unload/Query the Maya Vray Plugin.
+
+        Parameters:
+            load (bool): Load the VRay plugin.
+            unload (bool): Unload the VRay plugin.
+            query (bool): Query the status of the VRay plugin.
+        """
+
+        def is_loaded(plugin="vrayformaya.mll"):
+            return True if pm.pluginInfo(plugin, q=True, loaded=True) else False
+
+        if query:
+            return is_loaded()
+
+        vray = ["vrayformaya.mll", "vrayformayapatch.mll"]
+        try:
+            if load:
+                for plugin in vray:
+                    if not is_loaded(plugin):
+                        pm.loadPlugin(plugin)
+            if unload:
+                for plugin in vray:
+                    if is_loaded(plugin):
+                        pm.unloadPlugin(plugin)
+        except Exception as error:
+            print(error)
+
     @staticmethod
     def get_recent_files(index=None):
         """Get a list of recent files sorted by modification time.
@@ -209,7 +243,7 @@ class EnvUtils(ptk.HelpMixin):
         # Extend file data with modification times and filter invalid or autosave files
         file_data = []
         for f in files:
-            if ptk.is_valid(f) and "Autosave" not in f:
+            if ptk.is_valid(f, "file") and "Autosave" not in f:
                 try:
                     mod_time = os.path.getmtime(f)
                     file_data.append((f, mod_time))
@@ -255,12 +289,11 @@ class EnvUtils(ptk.HelpMixin):
             get_recent_projects(format='timestamp') --> Returns all recent projects in timestamp format.
             get_recent_projects(format='standard|timestamp') --> Returns a dictionary with standard paths as keys and timestamped paths as values.
         """
-        files = pm.optionVar(query="RecentProjectsList")
-        if not files:
+        dirs = pm.optionVar(query="RecentProjectsList")
+        if not dirs:
             return []
 
-        result = [ptk.format_path(f) for f in reversed(files) if ptk.is_valid(f)]
-
+        result = [ptk.format_path(d) for d in reversed(dirs) if ptk.is_valid(d, "dir")]
         if index is not None:
             try:
                 result = result[index]
@@ -367,28 +400,109 @@ class EnvUtils(ptk.HelpMixin):
 
     @staticmethod
     @ptk.filter_results
-    def get_workspace_scenes(full_path=True, recursive=False):
-        """Get a list of maya scene files from the current workspace directory.
+    def find_workspaces(
+        root_dir: str,
+        return_type: str = "dir",
+        ignore_empty: bool = True,
+        recursive: bool = True,
+    ) -> list:
+        """Recursively find Maya workspaces under a root directory.
+        A workspace is a folder containing 'workspace.mel'.
 
         Parameters:
-            full_path (bool): Return the full path instead of just the filename.
-            recursive (bool): Whether to return results from just the root directory.
+            root_dir (str): Folder to search from.
+            return_type (str): 'dir', 'dirname', 'dirname|dir', or 'dir|dirname'.
+            ignore_empty (bool): If True, only include workspaces that contain
+                                at least one .ma or .mb file inside the 'scenes/' folder.
+            recursive (bool): If True, search recursively for scene files within workspaces
+                             for validation. If False, only look in the direct 'scenes/' folder.
 
         Returns:
-            (list)
+            list: Filtered results in the requested format.
         """
-        workspace_dir = str(pm.workspace(q=True, rd=1))  # get current project path.
+        from pathlib import Path
+
+        results = []
+
+        # Walk through the root directory
+        for dirpath, dirnames, filenames in os.walk(root_dir):
+            # Check if workspace.mel exists in the directory
+            if "workspace.mel" not in filenames:
+                continue
+
+            dirpath = ptk.format_path(dirpath)
+            dirname = os.path.basename(dirpath)
+
+            if ignore_empty:
+                scenes_path = Path(dirpath) / "scenes"
+
+                # Only check for Maya scene files in the 'scenes' folder
+                if scenes_path.is_dir():
+                    # Search for scene files based on recursive setting
+                    if recursive:
+                        # Use rglob for recursive search for scene files
+                        scene_files = list(scenes_path.rglob("*.ma")) + list(
+                            scenes_path.rglob("*.mb")
+                        )
+                    else:
+                        # Use glob for non-recursive search (direct files only)
+                        scene_files = list(scenes_path.glob("*.ma")) + list(
+                            scenes_path.glob("*.mb")
+                        )
+
+                    # If Maya scene files are found, it's a valid workspace
+                    if scene_files:
+                        results.append((dirname, dirpath))
+                        continue  # Exit early after finding the first scene file
+
+            else:
+                results.append((dirname, dirpath))
+
+        # Handle return format (dir, dirname, or both)
+        if "|" in return_type:
+            a, b = return_type.split("|")
+            idx = {"dirname": 0, "dir": 1}
+            return [(r[idx[a]], r[idx[b]]) for r in results]
+
+        return [r[0] if return_type == "dirname" else r[1] for r in results]
+
+    @staticmethod
+    @ptk.filter_results
+    def get_workspace_scenes(
+        root_dir: Optional[str] = None,
+        full_path: bool = True,
+        recursive: bool = False,
+        omit_autosave: bool = True,
+        file_types=["*.ma", "*.mb"],
+    ) -> list[str]:
+        """Return a list of Maya scene files (.ma/.mb) from the given or current workspace directory.
+
+        Parameters:
+            root_dir (Optional[str]): Directory to scan. Defaults to current workspace.
+            full_path (bool): If True, returns full paths; else returns file names.
+            recursive (bool): Whether to include subdirectories.
+            omit_autosave (bool): Exclude autosave files like name.0001.ma
+            file_types (list[str]): List of file extensions to include, e.g., ['*.ma', '*.mb'].
+
+        Returns:
+            list[str]: Maya scene file paths or names.
+        """
+        import re
+
+        root_dir = root_dir or str(pm.workspace(q=True, rd=True))
 
         files = ptk.get_dir_contents(
-            workspace_dir, "filepath", recursive=recursive, inc_files=("*.mb", "*.ma")
+            root_dir,
+            content="filepath" if full_path else "file",
+            recursive=recursive,
+            inc_files=file_types,
         )
-        # Replace any backslashes with forward slashes.
-        result = [ptk.format_path(f) for f in files]
 
-        if not full_path:
-            result = [f.split("\\")[-1] for f in result]
+        if omit_autosave:
+            autosave_regex = re.compile(r".+\.\d{4}\.(ma|mb)$")
+            files = [f for f in files if not autosave_regex.match(os.path.basename(f))]
 
-        return result
+        return [ptk.format_path(f) for f in files]
 
     @classmethod
     def find_workspace_using_path(
@@ -403,7 +517,7 @@ class EnvUtils(ptk.HelpMixin):
             Optional[str]: The directory containing the workspace.mel file, or None if not found.
         """
         if scene_path is None:
-            scene_path = cls.get_maya_info("scene_path")
+            scene_path = cls.get_env_info("scene_path")
 
         dir_path = os.path.dirname(scene_path)
         while dir_path:
@@ -546,6 +660,83 @@ class EnvUtils(ptk.HelpMixin):
         except Exception as e:
             print(f"Failed to export scene as FBX: {str(e)}")
 
+    @staticmethod
+    def sanitize_namespace(namespace: str) -> str:
+        """Sanitize the namespace by replacing or removing illegal characters.
+
+        Parameters:
+            namespace (str): The namespace string to sanitize
+
+        Returns:
+            str: Sanitized namespace containing only valid characters
+        """
+        import re
+
+        return re.sub(r"[^a-zA-Z0-9_]", "_", namespace)
+
+    @staticmethod
+    def resolve_file_path_in_workspaces(
+        selected_file: str, workspace_files: dict
+    ) -> Optional[str]:
+        """Resolve a file name to its full path by searching in workspace files.
+
+        Parameters:
+            selected_file (str): The file name to resolve
+            workspace_files (dict): Dictionary mapping workspace paths to file lists
+
+        Returns:
+            Optional[str]: Full file path if found, None otherwise
+        """
+        return next(
+            (
+                fp
+                for files in workspace_files.values()
+                for fp in files
+                if os.path.basename(fp) == selected_file
+            ),
+            None,
+        )
+
+    @classmethod
+    def get_workspace_file_cache(cls, workspaces: list, recursive: bool = True) -> dict:
+        """Build a cache of workspace files for multiple workspaces.
+
+        Parameters:
+            workspaces (list): List of (dirname, workspace_path) tuples
+            recursive (bool): Whether to search recursively for scene files
+
+        Returns:
+            dict: Dictionary mapping workspace paths to their scene file lists
+        """
+        workspace_files = {}
+
+        for _, ws_path in workspaces:
+            if os.path.isdir(ws_path):
+                scenes = cls.get_workspace_scenes(
+                    root_dir=ws_path,
+                    full_path=True,
+                    recursive=recursive,
+                    omit_autosave=True,
+                )
+                workspace_files[ws_path] = scenes
+
+        return workspace_files
+
+    @staticmethod
+    def matches_autosave_pattern(filename: str) -> bool:
+        """Check if a file matches the Maya autosave pattern.
+
+        Parameters:
+            filename (str): The filename to check
+
+        Returns:
+            bool: True if the file matches autosave pattern, False otherwise
+        """
+        import re
+
+        autosave_regex = re.compile(r".+\.\d{4}\.(ma|mb)$")
+        return bool(autosave_regex.match(filename))
+
 
 # -----------------------------------------------------------------------------
 
