@@ -2,6 +2,14 @@
 # coding=utf-8
 
 from mayatk.env_utils.scene_exporter._scene_exporter import *
+from mayatk.env_utils.scene_exporter.task_manager import TaskManager
+from mayatk.env_utils.scene_exporter.task_factory import TaskFactory
+
+__all__ = [
+    "SceneExporter",
+    "TaskManager",
+    "TaskFactory",
+]
 
 # --------------------------------------------------------------------------------------------
 
