@@ -10,7 +10,6 @@ except ImportError as error:
 import pythontk as ptk
 
 # from this package:
-from mayatk.core_utils import CoreUtils
 from mayatk.node_utils import NodeUtils
 from mayatk.env_utils import EnvUtils
 
@@ -71,7 +70,7 @@ class GraphCollector:
         # Adding metadata for file nodes regarding their map type
         if node_type == "file":
             image_name = pm.getAttr(f"{node}.fileTextureName", "")
-            map_type = ptk.get_map_type_from_filename(image_name)
+            map_type = ptk.resolve_map_type(image_name)
             graph_info[placeholder_name]["metadata"]["map_type"] = map_type
 
     def _is_connected_to_shading_engine(self, node):
@@ -171,7 +170,7 @@ class GraphRestorer:
 
         # Dictionary to hold available map types and their paths
         available_map_types = {
-            ptk.get_map_type_from_filename(path): path for path in self.texture_paths
+            ptk.resolve_map_type(path): path for path in self.texture_paths
         }
 
         for placeholder, node_info in self.graph_config.items():
@@ -326,19 +325,21 @@ class QTextEditLogger(logging.Handler):
 
 
 class ShaderTemplatesSlots:
-    def __init__(self, **kwargs):
-        super().__init__(**kwargs)
-        self.sb = self.switchboard()
-        self.ui = self.sb.shader_templates
-        self.workspace_dir = EnvUtils.get_maya_info("workspace_dir")
+    def __init__(self, switchboard, log_level="WARNING"):
+        super().__init__()
+
+        self.sb = switchboard
+        self.ui = self.sb.loaded_ui.shader_templates
+
+        self.workspace_dir = EnvUtils.get_env_info("workspace_dir")
         self.source_images_dir = os.path.join(self.workspace_dir, "sourceimages")
         self.image_files = None
 
         # Setup logging
-        self.log = logging.getLogger("ShaderTemplateManager")
-        self.log.setLevel(logging.DEBUG)
+        self.logger = logging.getLogger("ShaderTemplateManager")
+        self.logger.setLevel(log_level)
         log_handler = QTextEditLogger(self.ui.txt001)
-        self.log.addHandler(log_handler)
+        self.logger.addHandler(log_handler)
 
         # Load plugins
         EnvUtils.load_plugin("shaderFXPlugin")  # Load Stingray plugin
@@ -350,18 +351,19 @@ class ShaderTemplatesSlots:
 
     def cmb002_init(self, widget):
         """Initialize the ComboBox for shader templates."""
-        widget.refresh = True
         if not widget.is_initialized:
+            widget.restore_state = True  # Enable state restore
+            widget.refresh_on_show = True  # Call this method on show
             widget.menu.mode = "context"
             widget.menu.setTitle("Template Options")
             widget.menu.add(
-                self.sb.Label,
+                self.sb.registered_widgets.Label,
                 setObjectName="lbl000",
                 setText="Rename",
                 setToolTip="Rename the current template.",
             )
             widget.menu.add(
-                self.sb.Label,
+                self.sb.registered_widgets.Label,
                 setObjectName="lbl001",
                 setText="Delete",
                 setToolTip="Delete the current template.",
@@ -467,19 +469,10 @@ class ShaderTemplatesSlots:
 # -----------------------------------------------------------------------------
 
 if __name__ == "__main__":
-    from uitk import Switchboard
+    from mayatk.ui_utils.ui_manager import UiManager
 
-    parent = CoreUtils.get_main_window()
-    ui_file = os.path.join(os.path.dirname(__file__), "shader_templates.ui")
-    sb = Switchboard(parent, ui_location=ui_file, slot_location=ShaderTemplatesSlots)
-
-    sb.current_ui.set_attributes(WA_TranslucentBackground=True)
-    sb.current_ui.set_flags(FramelessWindowHint=True, WindowStaysOnTopHint=True)
-    sb.current_ui.set_style(theme="dark", style_class="translucentBgWithBorder")
-    sb.current_ui.header.configure_buttons(
-        menu_button=True, minimize_button=True, hide_button=True
-    )
-    sb.current_ui.show(pos="screen", app_exec=True)
+    ui = UiManager.instance().get("shader_templates", reload=True)
+    ui.show(pos="screen", app_exec=True)
 
 # -----------------------------------------------------------------------------
 # Notes
