@@ -26,7 +26,7 @@ class StingrayArnoldShader:
     color_warning = "rgb(200, 200, 100)"
     color_error = "rgb(255, 100, 100)"
 
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def create_network(
         self,
         textures: List[str],
@@ -63,11 +63,11 @@ class StingrayArnoldShader:
         # Process each texture
         length = len(textures)
         progress = 0
-        base_dir = EnvUtils.get_maya_info("sourceimages")
+        base_dir = EnvUtils.get_env_info("sourceimages")
         for texture in ptk.convert_to_relative_path(textures, base_dir):
             progress += 1
             texture_name = ptk.format_path(texture, "file")
-            texture_type = ptk.get_map_type_from_filename(
+            texture_type = ptk.resolve_map_type(
                 texture,
             )
 
@@ -118,7 +118,7 @@ class StingrayArnoldShader:
         sr_node = NodeUtils.create_render_node("StingrayPBS", name=name)
 
         if opacity:
-            maya_install_path = EnvUtils.get_maya_info("install_path")
+            maya_install_path = EnvUtils.get_env_info("install_path")
 
             graph = os.path.join(
                 maya_install_path,
@@ -485,9 +485,19 @@ class StingrayArnoldShader:
                 created_roughness_map = ptk.create_roughness_from_spec(specular_map[0])
                 created_metallic_map = ptk.create_metallic_from_spec(specular_map[0])
 
-                # Combine the created roughness and metallic maps into a metallic smoothness map
+                # Save these images to disk and get their file paths
+                base_name = ptk.get_base_texture_name(specular_map[0])
+                out_dir = os.path.dirname(specular_map[0])
+
+                rough_path = os.path.join(out_dir, f"{base_name}_Roughness.png")
+                metal_path = os.path.join(out_dir, f"{base_name}_Metallic.png")
+
+                created_roughness_map.save(rough_path)
+                created_metallic_map.save(metal_path)
+
+                # Now you can combine using file paths:
                 combined_map = ptk.pack_smoothness_into_metallic(
-                    created_metallic_map, created_roughness_map, invert_alpha=True
+                    metal_path, rough_path, invert_alpha=True
                 )
 
                 # Remove individual metallic, roughness, smoothness maps and the newly created maps
@@ -608,12 +618,13 @@ class StingrayArnoldShaderSlots(StingrayArnoldShader):
     """
     msg_completed = '<br><hl style="color:rgb(0, 255, 255);"><b>COMPLETED.</b></hl>'
 
-    def __init__(self, **kwargs):
+    def __init__(self, switchboard):
         super().__init__()
 
-        self.sb = self.switchboard()
-        self.ui = self.sb.stingray_arnold_shader
-        self.workspace_dir = EnvUtils.get_maya_info("workspace_dir")
+        self.sb = switchboard
+        self.ui = self.sb.loaded_ui.stingray_arnold_shader
+
+        self.workspace_dir = EnvUtils.get_env_info("workspace_dir")
         self.source_images_dir = os.path.join(self.workspace_dir, "sourceimages")
         self.image_files = None
 
@@ -621,29 +632,6 @@ class StingrayArnoldShaderSlots(StingrayArnoldShader):
         self.ui.progressBar.setValue(0)
         # self.init_header_menu()
 
-    # def init_header_menu(self):
-    #     """Configure header menu"""
-    #     self.ui.header.menu.setTitle("OPTIONS")
-    #     self.ui.header.menu.add(
-    #         self.sb.PushButton,
-    #         setText="HDR Manager",
-    #         setObjectName="b002",
-    #     )
-
-    #     module = hdr_manager
-    #     slot_class = module.HdrManagerSlots
-
-    #     # Register and configure HDR Manager UI
-    #     self.sb.register("hdr_manager.ui", slot_class, base_dir=module)
-    #     ui = self.sb.hdr_manager
-    #     ui.set_attributes(WA_TranslucentBackground=True)
-    #     ui.set_flags(FramelessWindowHint=True, WindowStaysOnTopHint=True)
-    #     ui.set_style(theme="dark", style_class="translucentBgWithBorder")
-    #     ui.header.configure_buttons(hide_button=True)
-
-    #     # Connect button click to show HDR Manager
-    #     self.ui.header.menu.b002.clicked.connect(lambda: ui.show(pos="cursor"))
-
     @property
     def mat_name(self) -> str:
         """Get the mat name from the user input text field.
@@ -743,21 +731,10 @@ class StingrayArnoldShaderSlots(StingrayArnoldShader):
 # -----------------------------------------------------------------------------
 
 if __name__ == "__main__":
-    from uitk import Switchboard
+    from mayatk.ui_utils.ui_manager import UiManager
 
-    parent = CoreUtils.get_main_window()
-    ui_file = os.path.join(os.path.dirname(__file__), "stingray_arnold_shader.ui")
-    sb = Switchboard(
-        parent, ui_location=ui_file, slot_location=StingrayArnoldShaderSlots
-    )
-
-    sb.current_ui.set_attributes(WA_TranslucentBackground=True)
-    sb.current_ui.set_flags(FramelessWindowHint=True, WindowStaysOnTopHint=True)
-    sb.current_ui.set_style(theme="dark", style_class="translucentBgWithBorder")
-    sb.current_ui.header.configure_buttons(
-        menu_button=True, minimize_button=True, hide_button=True
-    )
-    sb.current_ui.show(pos="screen", app_exec=True)
+    ui = UiManager.instance().get("stingray_arnold_shader", reload=True)
+    ui.show(pos="screen", app_exec=True)
 
 # -----------------------------------------------------------------------------
 # Notes
