@@ -25,65 +25,117 @@ class NodeUtils(ptk.HelpMixin):
         Returns:
             (str/list) The node type. A list is always returned when 'objects' is given as a list.
         """
+        from mayatk import Components
+
         types = []
         for obj in pm.ls(objects):
             if cls.is_group(obj):
                 typ = "group"
             elif cls.is_locator(obj):
                 typ = "locator"
+            elif cls.is_mesh(obj):
+                typ = "mesh"
             else:
-                typ = core_utils.Components.get_component_type(obj)
+                typ = Components.get_component_type(obj)
             if not typ:
                 typ = pm.objectType(obj)
             types.append(typ)
+            print(obj.name(), typ)
 
         return ptk.format_return(types, objects)
 
     @staticmethod
-    def is_locator(objects):
-        """Determine if each of the given object(s) is a locator.
-        A locator is a transform node that has a shape node child.
-        The shape node defines the appearance and behavior of the locator.
+    def is_mesh(objects, filter: bool = False):
+        """Return True for each object that is a transform node with a mesh shape child.
 
         Parameters:
             objects (str/obj/list): The object(s) to query.
+            filter (bool): If True, return only the objects that are meshes.
 
         Returns:
-            (bool)(list) A list is always returned when 'objects' is given as a list.
+            (bool/list) A list of booleans indicating whether each object is a mesh.
+            If 'filter' is True, returns a list of objects that are meshes.
+        """
+        objs = pm.ls(objects, transforms=True)
+        result = [
+            isinstance(obj, pm.nt.Transform)
+            and any(
+                isinstance(s, pm.nt.Mesh) for s in obj.getShapes(noIntermediate=True)
+            )
+            for obj in objs
+        ]
+        if filter:
+            return [obj for obj, is_mesh in zip(objs, result) if is_mesh]
+        return ptk.format_return(result, objects)
+
+    @staticmethod
+    def is_locator(objects, filter: bool = False):
+        """Determine if each of the given object(s) is a locator.
+
+        Parameters:
+            objects (str/obj/list): The object(s) to query.
+            filter (bool): If True, return only the objects that are locators.
+
+        Returns:
+            (bool/list) A list of booleans indicating whether each object is a locator.
+            If 'filter' is True, returns a list of objects that are locators.
         """
         objs = pm.ls(objects, transforms=True)
-        # Get all locator shapes and their corresponding transforms
         locator_shapes = pm.ls(type="locator")
         locator_transforms = set(
             pm.listRelatives(locator_shapes, parent=True, path=True)
         )
-        # Determine if each object is a locator
         result = [obj in locator_transforms for obj in objs]
+        if filter:
+            return [obj for obj, is_loc in zip(objs, result) if is_loc]
         return ptk.format_return(result, objects)
 
     @staticmethod
-    def is_group(objects):
+    def is_group(objects, filter: bool = False):
         """Determine if each of the given object(s) is a group.
-        A group is defined as a transform node with children and no shape nodes directly beneath it.
 
         Parameters:
             objects (str/obj/list): The object(s) to query.
+            filter (bool): If True, return only the objects that are groups.
 
         Returns:
-            (bool)(list) A list is always returned when 'objects' is given as a list.
+            (bool/list) A list of booleans indicating whether each object is a group.
+            If 'filter' is True, returns a list of objects that are groups.
         """
+        objs = pm.ls(objects)
         result = []
-        for n in pm.ls(objects):
+        for n in objs:
             try:
-                # Check if the object is a transform node
                 is_transform = type(n) == pm.nodetypes.Transform
-                # Check if the transform node does not have any shape nodes directly beneath it
                 no_shapes = len(n.getShapes(noIntermediate=True)) == 0
                 q = is_transform and no_shapes
             except AttributeError:
                 q = False
             result.append(q)
+        if filter:
+            return [obj for obj, is_grp in zip(objs, result) if is_grp]
+        return ptk.format_return(result, objects)
 
+    @staticmethod
+    def is_geometry(objects, filter: bool = False):
+        """Return True for each object that has a shape node and is not a group.
+
+        Parameters:
+            objects (str/obj/list): The object(s) to query.
+            filter (bool): If True, return only the objects that are geometries.
+
+        Returns:
+            (bool/list) A list of booleans indicating whether each object is geometry.
+            If 'filter' is True, returns a list of objects that are geometries.
+        """
+        objs = pm.ls(objects, transforms=True)
+        result = [
+            isinstance(obj, pm.nt.Transform)
+            and bool(obj.getShapes(noIntermediate=True))
+            for obj in objs
+        ]
+        if filter:
+            return [obj for obj, is_geom in zip(objs, result) if is_geom]
         return ptk.format_return(result, objects)
 
     @classmethod
@@ -185,7 +237,7 @@ class NodeUtils(ptk.HelpMixin):
             (obj/list) Transform node(s) or node attributes. If 'nodes' is provided as a list, a list is always returned. If a single node is provided, a single object or a list, depending on the content, is returned.
         """
         result = []
-        for node in pm.ls(nodes, long=True):
+        for node in pm.ls(nodes, long=True, flatten=True):
             try:
                 # Check if node is a transform and directly add it
                 if isinstance(node, pm.nt.Transform):
@@ -235,7 +287,7 @@ class NodeUtils(ptk.HelpMixin):
             (obj/list) node(s) or node attributes. A list is always returned when 'nodes' is given as a list.
         """
         result = []
-        for node in pm.ls(nodes, long=True):
+        for node in pm.ls(nodes, long=True, flatten=True):
             shapes = pm.listRelatives(
                 node, children=1, shapes=1
             )  # get shape node from transform: returns list ie. [nt.Mesh('pConeShape1')]
@@ -277,7 +329,7 @@ class NodeUtils(ptk.HelpMixin):
             (obj/list) node(s) or node attributes. A list is always returned when 'nodes' is given as a list.
         """
         result = []
-        for node in pm.ls(nodes, long=True):
+        for node in pm.ls(nodes, long=True, flatten=True):
             shapes = pm.listRelatives(
                 node, children=1, shapes=1
             )  # get shape node from transform: returns list ie. [nt.Mesh('pConeShape1')]
@@ -371,7 +423,7 @@ class NodeUtils(ptk.HelpMixin):
             if name:
                 node.rename(name)
             if node:  # Set attributes if the node was created successfully
-                cls.set_node_attributes(node, option_quiet=False, **attributes)
+                cls.set_node_attributes(node, quiet=False, **attributes)
             return node
         except Exception as e:
             print(f"Failed to create node of type '{node_type}'. Error: {e}")
@@ -492,38 +544,50 @@ class NodeUtils(ptk.HelpMixin):
 
     @classmethod
     def set_node_attributes(
-        cls, node, option_create=False, option_quiet=False, **attributes
+        cls,
+        node,
+        create: bool = False,
+        quiet: bool = False,
+        keyable: bool = False,
+        lock: bool = False,
+        **attributes,
     ):
-        """Set node attribute values, with options to create attributes if they don't exist and to suppress errors.
+        """Set values on existing attributes. Optionally makes them keyable or locks them.
 
         Parameters:
             node (str/obj): The node to set attributes on.
-            option_create (bool): If True, creates the attribute if it doesn't exist. Defaults to False.
-            option_quiet (bool): If True, suppresses any errors encountered. Defaults to False.
-            **attributes: Arbitrary keyword arguments for attribute names and their values.
+            create (bool): If True, delegates missing attribute creation to `set_node_custom_attributes`.
+            quiet (bool): Suppress warnings on failure.
+            keyable (bool): Make attribute keyable if not already.
+            lock (bool): Lock the attribute after setting.
+            **attributes: Attr name and value pairs.
         """
+        if isinstance(node, str):
+            node = pm.PyNode(node)
+
         for attr, value in attributes.items():
-            attribute_name = f"{node}.{attr}"
-            try:  # Check if the attribute is locked
-                if pm.getAttr(attribute_name, lock=True):
-                    pm.warning(f"The attribute '{attribute_name}' is locked.")
+            try:
+                if node.attr(attr).isLocked():
+                    pm.warning(f"The attribute '{node}.{attr}' is locked.")
                     continue
 
-                # Set the attribute value
-                pm.setAttr(attribute_name, value)
+                pm.setAttr(
+                    node.attr(attr),
+                    value,
+                    keyable=keyable,
+                    lock=lock,
+                )
 
             except pm.MayaAttributeError:
-                if option_create:  # Attempt to create the attribute and set its value
-                    cls.set_node_custom_attributes(node, **{attr: value})
-                elif not option_quiet:
-                    pm.warning(
-                        f"Attribute '{attr}' does not exist on node '{node}', and 'option_create' is False."
+                if create:
+                    cls.set_node_custom_attributes(
+                        node, keyable=keyable, **{attr: value}
                     )
+                elif not quiet:
+                    pm.warning(f"Attribute '{attr}' does not exist on '{node}'.")
             except Exception as e:
-                if not option_quiet:
-                    pm.warning(
-                        f"Failed to set attribute '{attr}' on node '{node}'. Error: {str(e)}"
-                    )
+                if not quiet:
+                    pm.warning(f"Failed to set '{attr}' on '{node}': {e}")
 
     @classmethod
     def get_maya_attribute_type(cls, value):
@@ -590,12 +654,13 @@ class NodeUtils(ptk.HelpMixin):
                 return "matrix"
 
     @classmethod
-    def set_node_custom_attributes(cls, node, **attributes):
+    def set_node_custom_attributes(cls, node, keyable=True, **attributes):
         """Set node attribute values. If the attribute doesn't exist, it will be created.
 
         Parameters:
             node (str/obj): The node to set attributes of.
-            attributes (dict): Attributes and their corresponding value to set. ie. attribute_name=value
+            keyable (bool): If True, the attribute will be keyable. Defaults to True.
+            **attributes: Arbitrary keyword arguments for attribute names and their values.
         """
         if isinstance(node, str):
             node = pm.PyNode(node)
@@ -626,7 +691,7 @@ class NodeUtils(ptk.HelpMixin):
                         pm.setAttr(f"{node}.{component_name}", component)
                 else:
                     node.addAttr(
-                        attr, defaultValue=value, keyable=True, dataType=attr_type
+                        attr, defaultValue=value, keyable=keyable, dataType=attr_type
                     )
                     pm.setAttr(
                         f"{node}.{attr}", value
@@ -746,41 +811,84 @@ class NodeUtils(ptk.HelpMixin):
         return instances
 
     @classmethod
-    def convert_to_instances(cls, objects=[], append=""):
-        """The first selected object will be instanced across all other selected objects.
+    @core_utils.CoreUtils.undoable
+    def replace_with_instances(
+        cls,
+        objects=None,
+        append="",
+        freeze_transforms=False,
+        center_pivot=True,
+        delete_history=True,
+    ):
+        """Replace target objects with instances of the source object.
+
+        Takes the first object in the selection as the source and replaces all
+        subsequent objects with instances of that source object. The instances
+        inherit the transform and hierarchy of the replaced objects.
 
         Parameters:
-            objects (list): A list of objects to convert to instances. The first object will be the instance parent.
-            append (str): Append a string to the end of any instanced objects. ie. '_INST'
+            objects (list): List of objects where first is source, rest are targets.
+                           If None, uses current selection.
+            append (str): String to append to instance names.
+            freeze_transforms (bool): Whether to freeze transforms before instancing.
+            center_pivot (bool): Whether to center pivot before instancing.
+            delete_history (bool): Whether to delete history before instancing.
 
         Returns:
-            (list) The instanced objects.
+            list: The newly created instance objects.
         """
-        for obj in objects[1:]:
-            name = obj.name()
-            objParent = pm.listRelatives(obj, parent=1)
+        from mayatk import XformUtils
 
-            instance = pm.instance(objects[0])
+        objects = pm.ls(objects) or pm.ls(orderedSelection=True)
+        try:
+            source, targets = objects[0], objects[1:]
+        except IndexError:
+            pm.warning("Operation requires a selection of at least two objects.")
+            return
 
-            cls.uninstance(obj)
-            pm.makeIdentity(obj, apply=1, translate=1, rotate=0, scale=0)
+        if any((freeze_transforms, center_pivot, delete_history)):
+            XformUtils.freeze_transforms(
+                objects,
+                translate=freeze_transforms,
+                center_pivot=center_pivot,
+                delete_history=delete_history,
+                force=True,
+            )
 
-            # Move object to center of the last selected items bounding box # pm.xform(instance, translation=pos, worldSpace=1, relative=1) #move to the original objects location.
-            pm.matchTransform(instance, obj, position=1, rotation=1, scale=1, pivots=1)
-
-            try:
-                # Parent the instance under the original objects parent.
-                pm.parent(instance, objParent)
-            except RuntimeError:  # It is already a child of the parent.
-                pass
-
-            # Delete history for the object so that the namespace is cleared.
-            pm.delete(obj, constructionHistory=True)
-            pm.delete(obj)
+        new_instances = []
+        for target in targets:
+            name = target.name()
+            objParent = pm.listRelatives(target, parent=True)
+            instance = pm.instance(source)[0]  # returns a list
+            pm.matchTransform(
+                instance, target, position=True, rotation=True, scale=True, pivots=True
+            )
+            if objParent:
+                try:
+                    pm.parent(instance, objParent)
+                except RuntimeError:
+                    pass
             pm.rename(instance, name + append)
-        pm.select(objects[1:])
+            pm.delete(target)  # delete only the transform
+            new_instances.append(instance)
 
-        return objects[1:]
+        pm.select(new_instances)
+        return new_instances
+
+    @classmethod
+    def instance(cls, *args, **kwargs):
+        """Deprecated: Use replace_with_instances instead.
+
+        This method is kept for backward compatibility.
+        """
+        import warnings
+
+        warnings.warn(
+            "NodeUtils.instance() is deprecated. Use NodeUtils.replace_with_instances() instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
+        return cls.replace_with_instances(*args, **kwargs)
 
     @classmethod
     def uninstance(cls, objects):
@@ -801,6 +909,30 @@ class NodeUtils(ptk.HelpMixin):
                 pm.delete(obj)
                 pm.rename(duplicatedObject[0], obj)
 
+    @staticmethod
+    def filter_duplicate_instances(nodes) -> List["pm.PyNode"]:
+        """Keep only one transform per instance group.
+
+        Parameters:
+            nodes (str/obj/list): The nodes to filter.
+
+        Returns:
+            List[pm.PyNode]: Filtered list of nodes with unique instance groups.
+        """
+        transforms = NodeUtils.get_transform_node(nodes, returned_type="obj")
+        filtered = []
+        visited = set()
+        for t in transforms:
+            inst_group = NodeUtils.get_instances(t, return_parent_objects=True)
+            if not inst_group:
+                key = (t.longName(),)
+            else:
+                key = tuple(sorted(x.longName() for x in inst_group))
+            if key not in visited:
+                visited.add(key)
+                filtered.append(t)
+        return filtered
+
 
 # -----------------------------------------------------------------------------
 
