@@ -1,6 +1,6 @@
 # !/usr/bin/python
 # coding=utf-8
-from typing import List, Tuple, Union, Optional
+from typing import List, Tuple, Dict, Union, Optional
 
 try:
     import pymel.core as pm
@@ -22,6 +22,7 @@ class XformUtils(ptk.HelpMixin):
 
         Parameters:
             value (int/str): The axis value to convert, either an integer index or a string representation.
+                        Valid values are: 0 or "x", 1 or "-x", 2 or "y", 3 or "-y", 4 or "z", 5 or "-z".
             invert (bool): When True, inverts the axis direction.
             ortho (bool): When True, returns the axis that is orthogonal to the given axis.
             to_integer (bool): If True, returns the converted axis value as an integer index.
@@ -32,6 +33,13 @@ class XformUtils(ptk.HelpMixin):
         Raises:
             TypeError: If `value` is not an int or str.
             ValueError: If `value` is invalid.
+
+        Example:
+            convert_axis(0)  # Returns "x"
+            convert_axis("y")  # Returns "y"
+            convert_axis("x", invert=True)  # Returns "-x"
+            convert_axis(2, ortho=True)  # Returns "z"
+            convert_axis("z", to_integer=True) # Returns 4
         """
         index_to_axis = {0: "x", 1: "-x", 2: "y", 3: "-y", 4: "z", 5: "-z"}
         axis_to_index = {v: k for k, v in index_to_axis.items()}
@@ -115,7 +123,7 @@ class XformUtils(ptk.HelpMixin):
                 pm.xform(src, translation=target_pos, worldSpace=True)
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def drop_to_grid(
         objects, align="Mid", origin=False, center_pivot=False, freeze_transforms=False
     ):
@@ -198,7 +206,7 @@ class XformUtils(ptk.HelpMixin):
 
     @staticmethod
     @CoreUtils.selected
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def scale_connected_edges(objects, scale_factor=1.1) -> None:
         """Scales each set of connected edges separately, either uniformly or non-uniformly.
 
@@ -258,7 +266,7 @@ class XformUtils(ptk.HelpMixin):
                 vertex.setPosition(new_pos, space="world")
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def store_transforms(objects, prefix="original"):
         for obj in pm.ls(objects, type="transform"):
             # Store the world matrix and pivot points
@@ -280,57 +288,49 @@ class XformUtils(ptk.HelpMixin):
             pm.setAttr(f"{obj}.{prefix}_scalePivot", type="double3", *scale_pivot)
 
     @classmethod
-    @CoreUtils.undo
-    def freeze_transforms(cls, objects, center_pivot=False, force=False, **kwargs):
-        """Freezes the transformations of the specified objects.
+    @CoreUtils.undoable
+    def freeze_transforms(
+        cls, objects, center_pivot=False, force=True, delete_history=False, **kwargs
+    ):
+        """Freezes transformations on the given objects.
 
         Parameters:
-            objects (list): List of objects to freeze transformations on.
-            center_pivot (bool, optional): If True, centers the pivot of the objects. Default is False.
-            force (bool, optional): If True, unlocks any locked transform attributes and relocks them after the operation. Default is False.
-            **kwargs: Additional arguments passed to pm.makeIdentity.
-
-        Returns:
-            None
+            objects (list): List of transform nodes.
+            center_pivot (bool): If True, centers the pivot.
+            force (bool): If True, unlocks locked transform attributes and restores them after.
+            delete_history (bool): If True, deletes construction history after freeze.
+            **kwargs: Passed to pm.makeIdentity (e.g., t=True, r=True, s=True, n=0)
         """
-        for obj in pm.ls(objects, type="transform"):
+        from mayatk.rig_utils import RigUtils
+
+        objects = pm.ls(objects, type="transform", long=True)
+
+        lock_state: Dict[str, Dict[str, bool]] = {}
+
+        for obj in objects:
             if center_pivot:
-                pm.xform(objects, centerPivots=True)
-
-            if not pm.hasAttr(obj, "original_worldMatrix"):
-                cls.store_transforms(obj)
+                pm.xform(obj, centerPivots=True)
 
+            # Store lock state and unlock if force
             if force:
-                transform_attrs = [
-                    "translateX",
-                    "translateY",
-                    "translateZ",
-                    "rotateX",
-                    "rotateY",
-                    "rotateZ",
-                    "scaleX",
-                    "scaleY",
-                    "scaleZ",
-                ]
+                lock_state[obj.name()] = RigUtils.get_attr_lock_state(obj)
+                RigUtils.set_attr_lock_state(
+                    obj, translate=False, rotate=False, scale=False
+                )
 
-                locked_attrs = {
-                    attr: pm.getAttr(f"{obj}.{attr}", lock=True)
-                    for attr in transform_attrs
-                    if pm.getAttr(f"{obj}.{attr}", lock=True)
-                }
-                if locked_attrs:
-                    pm.setAttr(
-                        [f"{obj}.{attr}" for attr in locked_attrs.keys()], lock=False
-                    )
+            # Delete history if requested
+            if delete_history:
+                pm.delete(obj, constructionHistory=True)
 
-            # Freeze transformations to reset them
+            # Freeze transforms
             pm.makeIdentity(obj, apply=True, **kwargs)
 
-            if force and locked_attrs:
-                pm.setAttr([f"{obj}.{attr}" for attr in locked_attrs.keys()], lock=True)
+            # Restore lock state if needed
+            if force and obj.name() in lock_state:
+                RigUtils.set_attr_lock_state(obj, **lock_state[obj.name()])
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def restore_transforms(objects, prefix="original"):
         for obj in pm.ls(objects, type="transform"):
             # Check if the transform attributes are at their default values
@@ -362,7 +362,7 @@ class XformUtils(ptk.HelpMixin):
             pm.xform(obj, scalePivot=scale_pivot, worldSpace=True)
 
     @classmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def reset_translation(cls, objects):
         """Reset the translation transformations on the given object(s).
 
@@ -370,10 +370,10 @@ class XformUtils(ptk.HelpMixin):
             objects (str/obj/list): The object(s) to reset the translation values for.
         """
         for obj in pm.ls(objects):
-            pos = pm.objectCenter(obj)  # get the object's current position.
+            pos = pm.objectCenter(obj)  # Get the object's current position.
             # Move to origin and center pivot.
             cls.drop_to_grid(obj, origin=1, center_pivot=1)
-            pm.makeIdentity(obj, apply=1, t=1, r=0, s=0, n=0, pn=1)  # bake transforms
+            pm.makeIdentity(obj, apply=1, t=1, r=0, s=0, n=0, pn=1)  # Bake transforms
             # Move the object back to it's original position.
             pm.xform(obj, translation=pos)
 
@@ -391,47 +391,119 @@ class XformUtils(ptk.HelpMixin):
         pm.xform(node, translation=[x, y, z])
 
     @staticmethod
-    def get_manip_pivot_matrix(obj: Union[str, object, list]) -> "pm.datatypes.Matrix":
-        """Retrieves the current manipulator pivot's position and orientation in world space
-        and returns it as a transformation matrix.
+    def get_manip_pivot_matrix(
+        obj: Union[str, object, list], **kwargs
+    ) -> "pm.datatypes.Matrix":
+        """Return the object's transform matrix using xform, allowing kwargs override.
+
+        Parameters:
+            obj (str/object/list): Object to query.
+            **kwargs: Passed directly to pm.xform() in query mode.
 
         Returns:
-            obj (str/obj/list): The object to retrieve the manipulator pivot matrix for.
-            pm.datatypes.Matrix: A transformation matrix representing the manipulator pivot's position
-                                and orientation in world space.
+            pm.datatypes.Matrix: The resulting transformation matrix.
         """
-        # Query the object's world space matrix
-        world_matrix = pm.xform(obj, q=True, ws=True, m=True)
-
-        # Convert the world matrix to a Maya Matrix and return it
-        return pm.datatypes.Matrix(world_matrix)
+        matrix = pm.xform(obj, q=True, matrix=True, **kwargs)
+        return pm.datatypes.Matrix(matrix)
 
     @staticmethod
     def set_manip_pivot_matrix(
-        obj: Union[str, object, list], matrix: "pm.datatypes.Matrix"
+        obj: Union[str, object, list],
+        matrix: "pm.datatypes.Matrix",
+        **kwargs,
     ) -> None:
-        """Sets the manipulator pivot's position and orientation in world space based on
-        the provided transformation matrix.
+        """Apply a transformation matrix's position and orientation to the manip pivot.
 
         Parameters:
-            obj (str/obj/list): The object to set the manipulator pivot for.
-            matrix (pm.datatypes.Matrix): A transformation matrix containing the position and
-                                        orientation to set for the manipulator pivot.
+            obj (str/object/list): Object to set pivot on.
+            matrix (pm.datatypes.Matrix): Source matrix.
+            **kwargs: Passed directly to pm.manipPivot().
         """
-        # Extract translation and rotation components from the transformation matrix
-        transform_matrix = pm.datatypes.TransformationMatrix(matrix)
-        pos = transform_matrix.getTranslation(pm.datatypes.Space.kWorld)
-        ori_rotation = transform_matrix.eulerRotation()
+        tm = pm.datatypes.TransformationMatrix(matrix)
+        pos = tm.getTranslation(pm.datatypes.Space.kWorld)
+        rot = [pm.util.degrees(a) for a in tm.eulerRotation()]
 
-        # Convert rotation from radians to degrees
-        ori_degrees = [pm.util.degrees(angle) for angle in ori_rotation]
-
-        # Set the manipulator pivot position and orientation
         pm.select(obj, replace=True)
-        pm.manipPivot(p=pos, o=ori_degrees)
+        pm.manipPivot(p=pos, o=rot, **kwargs)
+
+    @classmethod
+    def get_operation_axis_pos(cls, node, pivot, axis_index=None):
+        """Determines the pivot position for mirroring/cutting along a specified axis or all axes.
+
+        Parameters:
+            node (PyNode): The object whose reference position is determined.
+            pivot (int/str/tuple/list): Mode or explicit position.
+                - `0` or `"boundingBox"` → Uses bounding box properties.
+                - `1` or `"object"` → Uses the object's rotate pivot.
+                - `2` or `"world"` → Uses world origin `(0,0,0)`.
+                - `"manip"` → Uses the manipulation pivot (scale pivot).
+                - `"center"` → Uses bounding box center.
+                - `"xmin"`, `"xmax"`, etc. → Uses specific bounding box limits.
+                - `"baked"` → Uses the baked (original) rotate pivot in world space.
+                - `(x, y, z)` → Uses a specified world-space pivot.
+            axis_index (int or None): Axis index (0=X, 1=Y, 2=Z). If `None`, returns full (x, y, z) list.
+
+        Returns:
+            float or list: The computed pivot position (single float if `axis_index` is specified, list if `None`).
+        """
+        # Return full vector if axis_index is None
+        if axis_index is None:
+            return [
+                cls.get_operation_axis_pos(node, pivot, 0),
+                cls.get_operation_axis_pos(node, pivot, 1),
+                cls.get_operation_axis_pos(node, pivot, 2),
+            ]
+
+        # Explicit world-space point
+        if isinstance(pivot, (tuple, list)) and len(pivot) == 3:
+            return float(pivot[axis_index])
+
+        # Object pivot (rotate pivot in world space)
+        if pivot in {1, "object"}:
+            obj_pivot_ws = pm.xform(node, q=True, ws=True, rp=True)
+            return float(obj_pivot_ws[axis_index])
+
+        # Manipulation pivot (scale pivot in world space)
+        if pivot == "manip":
+            manip_pivot_ws = pm.xform(node, q=True, ws=True, sp=True)
+            return float(manip_pivot_ws[axis_index])
+
+        # Baked pivot (local-space rotate pivot transformed to world space)
+        if pivot == "baked":
+            local_rp = pm.xform(node, q=True, rp=True, os=True)
+            world_rp = pm.dt.Point(local_rp) * pm.PyNode(node).getMatrix(
+                worldSpace=True
+            )
+            return float(world_rp[axis_index])
+
+        # World origin
+        if pivot in {2, "world"}:
+            return 0.0
+
+        # Bounding box center
+        if pivot == "center":
+            center = cls.get_bounding_box(node, "center")
+            return float(center[axis_index])
+
+        # Bounding box limits (xmin, ymax, etc.)
+        axis_map = {"xmin": 0, "xmax": 0, "ymin": 1, "ymax": 1, "zmin": 2, "zmax": 2}
+        valid_keys = cls.get_bounding_box(node, return_valid_keys=True)
+
+        if isinstance(pivot, str) and pivot in valid_keys:
+            val = cls.get_bounding_box(node, pivot)
+            if isinstance(val, (tuple, list)):
+                return float(val[axis_map[pivot]])
+            return float(val)
+
+        # Fallback to center
+        pm.warning(
+            f"Invalid pivot type '{pivot}' for {node}. Defaulting to bounding box center."
+        )
+        fallback = cls.get_bounding_box(node, "center")
+        return float(fallback[axis_index])
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def align_pivot_to_selection(align_from=[], align_to=[], translate=True):
         """Align one objects pivot point to another using 3 point align.
 
@@ -482,7 +554,7 @@ class XformUtils(ptk.HelpMixin):
 
     @staticmethod
     def reset_pivot_transforms(
-        objects: Optional[List[Union[str, object]]] = None
+        objects: Optional[List[Union[str, object]]] = None,
     ) -> None:
         """Reset Pivot Transforms for the specified objects or selected objects.
 
@@ -503,124 +575,95 @@ class XformUtils(ptk.HelpMixin):
             pm.manipPivot(obj, rotatePivot=True, scalePivot=True)
 
     @staticmethod
-    @CoreUtils.undo
-    def bake_pivot(
-        objects: list[str],
-        position: bool = False,
-        orientation: bool = False,
-        delete_history: bool = True,
-        preserve_normals: bool = False,
-    ) -> None:
-        """Bakes the pivot of the given objects.
+    @CoreUtils.undoable
+    def bake_pivot(objects, position=False, orientation=False):
+        """Bake the pivot orientation and position of the given object(s).
 
         Parameters:
-            objects (list[str]): List of objects to bake the pivot for.
+            objects (str/obj/list): The object(s) to bake the pivot orientation and position for.
             position (bool): Whether to bake the pivot position.
-            orientation (bool): Whether to bake the pivot orientation.
-            delete_history (bool): Delete Non-deformer history on the given object(s) before performing the operation.
-            preserve_normals (bool): Temporarily duplicate mesh and transfer normals back to the original mesh after baking the pivot.
+            orientation (bool): Whether to bake the pivot orientation
         """
-        if delete_history:
-            pm.bakePartialHistory(objects, prePostDeformers=True)
-
-        transforms = pm.ls(objects, objectsOnly=True, transforms=True, flatten=True)
-        shapes = NodeUtils.get_shape_node(transforms)
+        transforms = pm.ls(objects, transforms=1)
+        shapes = pm.ls(objects, shapes=1)
         objects = transforms + pm.listRelatives(
-            shapes, path=True, parent=True, type="transform"
+            shapes, path=1, parent=1, type="transform"
         )
 
         ctx = pm.currentCtx()
-        pivotModeActive = False
-        customModeActive = False
-        customOri = []
-
-        if ctx in ("RotateSuperContext", "manipRotateContext"):
-            customOri = pm.manipRotateContext("Rotate", q=True, orientAxes=True)
-            pivotModeActive = pm.manipRotateContext(
-                "Rotate", q=True, editPivotMode=True
-            )
-            customModeActive = pm.manipRotateContext("Rotate", q=True, mode=True) == 3
-        elif ctx in ("scaleSuperContext", "manipScaleContext"):
-            customOri = pm.manipScaleContext("Scale", q=True, orientAxes=True)
-            pivotModeActive = pm.manipScaleContext("Scale", q=True, editPivotMode=True)
-            customModeActive = pm.manipScaleContext("Scale", q=True, mode=True) == 6
-        else:
-            customOri = pm.manipMoveContext("Move", q=True, orientAxes=True)
-            pivotModeActive = pm.manipMoveContext("Move", q=True, editPivotMode=True)
-            customModeActive = pm.manipMoveContext("Move", q=True, mode=True) == 6
+        pivotModeActive = 0
+        customModeActive = 0
+        if ctx in ("RotateSuperContext", "manipRotateContext"):  # Rotate tool
+            customOri = pm.manipRotateContext("Rotate", q=1, orientAxes=1)
+            pivotModeActive = pm.manipRotateContext("Rotate", q=1, editPivotMode=1)
+            customModeActive = pm.manipRotateContext("Rotate", q=1, mode=1) == 3
+        elif ctx in ("scaleSuperContext", "manipScaleContext"):  # Scale tool
+            customOri = pm.manipScaleContext("Scale", q=1, orientAxes=1)
+            pivotModeActive = pm.manipScaleContext("Scale", q=1, editPivotMode=1)
+            customModeActive = pm.manipScaleContext("Scale", q=1, mode=1) == 6
+        else:  # use the move tool orientation
+            customOri = pm.manipMoveContext(
+                "Move", q=1, orientAxes=1
+            )  # get custom orientation
+            pivotModeActive = pm.manipMoveContext("Move", q=1, editPivotMode=1)
+            customModeActive = pm.manipMoveContext("Move", q=1, mode=1) == 6
 
         if orientation and customModeActive:
             if not position:
                 pm.mel.error(
-                    pm.mel.uiRes("m_bakeCustomToolPivot.kWrongAxisOriToolError")
+                    (pm.mel.uiRes("m_bakeCustomToolPivot.kWrongAxisOriToolError"))
                 )
                 return
 
             from math import degrees
 
-            customOri = [degrees(ori) for ori in customOri]
-            pm.rotate(objects, *customOri, a=True, pcp=True, pgp=True, ws=True, fo=True)
+            cX, cY, cZ = customOri = [
+                degrees(customOri[0]),
+                degrees(customOri[1]),
+                degrees(customOri[2]),
+            ]
+
+            pm.rotate(
+                objects, cX, cY, cZ, a=1, pcp=1, pgp=1, ws=1, fo=1
+            )  # Set object(s) rotation to the custom one (preserving child transform positions and geometry positions)
 
         if position:
-            try:
-                for obj in objects:
-                    m = pm.xform(obj, q=True, m=True)
-                    p = pm.xform(obj, q=True, os=True, sp=True)
-                    oldPivot = [
-                        p[0] * m[0] + p[1] * m[4] + p[2] * m[8] + m[12],
-                        p[0] * m[1] + p[1] * m[5] + p[2] * m[9] + m[13],
-                        p[0] * m[2] + p[1] * m[6] + p[2] * m[10] + m[14],
-                    ]
+            for obj in objects:
+                # Get pivot in parent space
+                m = pm.xform(obj, q=1, m=1)
+                p = pm.xform(obj, q=1, os=1, sp=1)
+                oldX, oldY, oldZ = [
+                    (p[0] * m[0] + p[1] * m[4] + p[2] * m[8] + m[12]),
+                    (p[0] * m[1] + p[1] * m[5] + p[2] * m[9] + m[13]),
+                    (p[0] * m[2] + p[1] * m[6] + p[2] * m[10] + m[14]),
+                ]
 
-                    pm.xform(obj, zeroTransformPivots=True)
-                    newPivot = pm.getAttr(f"{obj.name()}.translate")
-                    pm.move(
-                        obj,
-                        oldPivot[0] - newPivot[0],
-                        oldPivot[1] - newPivot[1],
-                        oldPivot[2] - newPivot[2],
-                        pcp=True,
-                        pgp=True,
-                        ls=True,
-                        r=True,
-                    )
-            except RuntimeError as e:
-                pm.displayWarning(f"Error during pivot position processing: {str(e)}")
-                return
+                pm.xform(obj, zeroTransformPivots=1)  # Zero out pivots
+
+                # Translate obj(s) back to previous pivot (preserving child transform positions and geometry positions)
+                newX, newY, newZ = pm.getAttr(
+                    obj.name() + ".translate"
+                )  # obj.translate
+                pm.move(
+                    obj, oldX - newX, oldY - newY, oldZ - newZ, pcp=1, pgp=1, ls=1, r=1
+                )
 
         if pivotModeActive:
-            pm.ctxEditMode()
+            pm.ctxEditMode()  # exit pivot mode
 
+        # Set the axis orientation mode back to obj
         if orientation and customModeActive:
             if ctx in ("RotateSuperContext", "manipRotateContext"):
                 pm.manipPivot(rotateToolOri=0)
             elif ctx in ("scaleSuperContext", "manipScaleContext"):
                 pm.manipPivot(scaleToolOri=0)
-            else:
+            else:  # Some other tool #Set move tool to obj mode and clear the custom ori. (so the tool won't restore it when activated)
                 pm.manipPivot(moveToolOri=0)
                 if ctx not in ("moveSuperContext", "manipMoveContext"):
-                    pm.manipPivot(ro=True)
-
-        if preserve_normals:
-            temp_objects = []
-            try:
-                # Create temporary duplicates for normals preservation
-                for obj in objects:
-                    temp_obj = pm.duplicate(obj, name=f"{obj}_tempForNormals")[0]
-                    temp_objects.append(temp_obj)
-
-                # Transfer normals from temp objects back to original objects
-                for original, temp in zip(objects, temp_objects):
-                    components.Components.transfer_normals(
-                        [temp.name(), original.name()]
-                    )
-            except RuntimeError as e:
-                pm.displayWarning(f"Error during normals transfer: {str(e)}")
-            finally:  # Always clean up temporary objects
-                pm.delete(temp_objects)
+                    pm.manipPivot(ro=1)
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def transfer_pivot(
         objects: List[Union[str, object]],
         translate: bool = False,
@@ -689,7 +732,7 @@ class XformUtils(ptk.HelpMixin):
                 pm.select(targets, replace=True)
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def aim_object_at_point(objects, target_pos, aim_vect=(1, 0, 0), up_vect=(0, 1, 0)):
         """Aim the given object(s) at the given world space position.
 
@@ -715,8 +758,47 @@ class XformUtils(ptk.HelpMixin):
 
         pm.delete(const, target)
 
+    @staticmethod
+    def orient_to_vector(
+        transform: "pm.nodetypes.Transform",
+        aim_vector: "pm.datatypes.Vector" = (1, 0, 0),
+        up_vector: "pm.datatypes.Vector" = (0, 1, 0),
+    ):
+        """Orients a transform so its local +X aims along the given world-space vector."""
+        transform = NodeUtils.get_transform_node(transform)
+        if not transform:
+            raise ValueError(f"// Error: Invalid transform node: {transform}")
+
+        up_vector_X, up_vector_Y, up_vector_Z = up_vector
+        up_vector = pm.datatypes.Vector(up_vector_X, up_vector_Y, up_vector_Z)
+
+        aim_vector_X, aim_vector_Y, aim_vector_Z = aim_vector
+        aim_vector = pm.datatypes.Vector(aim_vector_X, aim_vector_Y, aim_vector_Z)
+
+        temp = pm.spaceLocator()
+        target = pm.spaceLocator()
+
+        pos = transform.getTranslation(space="world")
+        temp.setTranslation(pos, space="world")
+        target.setTranslation(pos + aim_vector, space="world")
+
+        pm.delete(
+            pm.aimConstraint(
+                target,
+                temp,
+                aimVector=(1, 0, 0),
+                upVector=up_vector,
+                worldUpType="vector",
+                worldUpVector=up_vector,
+                maintainOffset=False,
+            )
+        )
+
+        transform.setRotation(temp.getRotation(space="world"), space="world")
+        pm.delete([temp, target])
+
     @classmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def rotate_axis(cls, objects, target_pos):
         """Aim the given object at the given world space position.
         All rotations in rotated channel, geometry is transformed so
@@ -824,69 +906,77 @@ class XformUtils(ptk.HelpMixin):
         return center_pos
 
     @staticmethod
-    def get_bounding_box(objects, value="", world_space=True):
+    def get_bounding_box(objects, value="", world_space=True, return_valid_keys=False):
         """Calculate and retrieve specific properties of the bounding box for the given object(s) or component(s).
 
-        The method computes the bounding box that encompasses all specified objects or components.
-        It can return various properties of this bounding box, such as its minimum and maximum extents,
-        its size along each axis, its total volume, and the central point. The properties to return
-        are specified as strings within the 'value' parameter, which can include multiple values separated by
-        a pipe ('|') character. The calculations can be performed in either world or local object space.
-
         Parameters:
-            objects (str/obj/list): The object(s) or component(s) to query. This can be a single object
-                                    or component, or a list of objects/components.
-            value (str): A string representing the specific bounding box data to return. This can include
-                         multiple properties separated by '|'. Valid options (case insensitive) are:
-                         'xmin', 'xmax', 'ymin', 'ymax', 'zmin', 'zmax', 'size', 'sizex', 'sizey',
-                         'sizez', 'volume', 'center', 'centroid', 'minsize', and 'maxsize'.
-            world_space (bool): If True, calculates the bounding box in world space. If False, uses local
-                                object space. Default is True.
+            objects (str/obj/list): The object(s) or component(s) to query.
+            value (str): A string representing the specific bounding box data to return.
+                        Multiple properties can be requested using '|'.
+            world_space (bool): If True, computes the bounding box in world space.
+            return_valid_keys (bool): If True, returns all valid bbox keys instead of computing the bbox.
+
         Returns:
-            float/tuple: The requested bounding box value(s). If a single value is requested, a float is
-                         returned. If multiple values are requested, a tuple of floats is returned.
-        Raises:
-            ValueError: If no objects are provided, if an invalid 'value' is specified, or if other input
-                        parameters are incorrect.
-        Examples:
-            # To get the size of the bounding box:
-            size = YourClassNameHere.get_bounding_box(obj, "size")
-            # To get the x, y, and z sizes individually:
-            sizex, sizey, sizez = YourClassNameHere.get_bounding_box(obj, "sizex|sizey|sizez")
+            float/tuple/list: The requested bounding box value(s), or a list of valid keys if `return_valid_keys=True`.
         """
+        bbox_values = {
+            "xmin": None,
+            "xmax": None,
+            "ymin": None,
+            "ymax": None,
+            "zmin": None,
+            "zmax": None,
+            "sizex": None,
+            "sizey": None,
+            "sizez": None,
+            "size": None,
+            "volume": None,
+            "center": None,
+            "centroid": None,
+            "minsize": None,
+            "maxsize": None,
+        }
+
+        # Return only the valid keys if requested
+        if return_valid_keys:
+            return list(bbox_values.keys())
+
         # Validate input objects
         if not objects:
             raise ValueError("No objects provided for bounding box calculation.")
 
         objs = objects if isinstance(objects, (list, tuple)) else [objects]
+        bbox = (
+            pm.exactWorldBoundingBox(objs)
+            if world_space
+            else pm.xform(objs, q=True, bb=True, ws=False)
+        )
 
-        if world_space:
-            bbox = pm.exactWorldBoundingBox(objs)
-        else:
-            bbox = pm.xform(objs, q=True, bb=True, ws=False)
-
+        # Assign real values to the dictionary
         xmin, ymin, zmin, xmax, ymax, zmax = bbox
         size = (xmax - xmin, ymax - ymin, zmax - zmin)
         center = ((xmin + xmax) / 2, (ymin + ymax) / 2, (zmin + zmax) / 2)
         volume = size[0] * size[1] * size[2]
 
-        bbox_values = {
-            "xmin": xmin,
-            "xmax": xmax,
-            "ymin": ymin,
-            "ymax": ymax,
-            "zmin": zmin,
-            "zmax": zmax,
-            "sizex": size[0],
-            "sizey": size[1],
-            "sizez": size[2],
-            "size": size,
-            "volume": volume,
-            "center": center,
-            "centroid": center,
-            "minsize": min(size),
-            "maxsize": max(size),
-        }
+        bbox_values.update(
+            {
+                "xmin": xmin,
+                "xmax": xmax,
+                "ymin": ymin,
+                "ymax": ymax,
+                "zmin": zmin,
+                "zmax": zmax,
+                "sizex": size[0],
+                "sizey": size[1],
+                "sizez": size[2],
+                "size": size,
+                "volume": volume,
+                "center": center,
+                "centroid": center,
+                "minsize": min(size),
+                "maxsize": max(size),
+            }
+        )
 
         values = value.lower().split("|")
         try:
@@ -1194,7 +1284,7 @@ class XformUtils(ptk.HelpMixin):
         return ordered_objs
 
     @staticmethod
-    @CoreUtils.undo
+    @CoreUtils.undoable
     def align_vertices(mode, average=False, edgeloop=False):
         """Align vertices.
 
